"""Helper functions to create Ladybug charts."""

from __future__ import annotations
from enum import Enum
from typing import List, Tuple
from ladybug.datacollection import HourlyContinuousCollection, \
    HourlyDiscontinuousCollection, MonthlyPerHourCollection, MonthlyCollection
from ladybug.color import Color, Colorset
from ladybug.header import Header
from ladybug.analysisperiod import AnalysisPeriod
from ladybug_geometry.geometry2d import Mesh2D
from ladybug_geometry.geometry2d.pointvector import Point2D


def discontinuous_to_continuous(
        data: HourlyDiscontinuousCollection) -> Tuple[HourlyContinuousCollection,
                                                      List[float, float]]:
    """Fill the gaps in Discontinuous data to create Continuous data.

    Args:
        data: A Ladybug HourlyDiscontinuousCollection.

    Returns:
        A tuple of two items

        -   A Ladybug HourlyContinuousCollection.

        -   A List of two floats representing min and max values of the data.
    """

    assert isinstance(data, HourlyDiscontinuousCollection), 'Only' \
        f' HourlyDiscontinuousCollection is supported. instead got {type(data)}'

    hoys = [dt.hoy for dt in data.datetimes]
    hoy_val = dict(zip(hoys, data.values))
    # We need to find the range of the data here before we inject None values
    data_range = [min(hoy_val.values()), max(hoy_val.values())]

    values = []
    for count in range(8760):
        if count in hoy_val:
            values.append(hoy_val[count])
        else:
            values.append(None)

    header = Header(data.header.data_type, data.header.unit,
                    AnalysisPeriod(), data.header.metadata)

    return HourlyContinuousCollection(header, values), data_range


def rgb_to_hex(color: Color) -> str:
    """Convert a RGB color to Hex color."""

    return f'#{color.r:02X}{color.g:02X}{color.b:02X}'


class ColorSet(Enum):
    """Colors for a legend."""
    annual_comfort = 'annual_comfort'
    benefit = 'benefit'
    benefit_harm = 'benefit_harm'
    black_to_white = 'black_to_white'
    blue_green_red = 'blue_green_red'
    cloud_cover = 'cloud_cover'
    cold_sensation = 'cloud_sensation'
    ecotect = 'ecotect'
    energy_balance = 'energy_balance'
    energy_balance_storage = 'energy_balance_storag'
    glare_study = 'glare_study'
    harm = 'harm'
    heat_sensation = 'heat_sensation'
    multi_colored = 'multi_colored'
    multicolored_2 = 'multicolored_2'
    multicolored_3 = 'multicolored_3'
    nuanced = 'nuanced'
    openstudio_palette = 'openstudio_palette'
    original = 'original'
    peak_load_balance = 'peak_load_balance'
    shade_benefit = 'shade_benefit'
    shade_benefit_harm = 'shade_benefit_harm'
    shade_harm = 'shade_harm'
    shadow_study = 'shadow_study'
    therm = 'therm'
    thermal_comfort = 'thermal_comfort'
    view_study = 'view_study'


color_set = {
    'annual_comfort': Colorset.annual_comfort(),
    'benefit': Colorset.benefit(),
    'benefit_harm': Colorset.benefit_harm(),
    'black_to_white': Colorset.black_to_white(),
    'blue_green_red': Colorset.blue_green_red(),
    'cloud_cover': Colorset.cloud_cover(),
    'cold_sensation': Colorset.cold_sensation(),
    'ecotect': Colorset.ecotect(),
    'energy_balance': Colorset.energy_balance(),
    'energy_balance_storage': Colorset.energy_balance_storage(),
    'glare_study': Colorset.glare_study(),
    'harm': Colorset.harm(),
    'heat_sensation': Colorset.heat_sensation(),
    'multi_colored': Colorset.multi_colored(),
    'multicolored_2': Colorset.multicolored_2(),
    'multicolored_3': Colorset.multicolored_3(),
    'nuanced': Colorset.nuanced(),
    'openstudio_palette': Colorset.openstudio_palette(),
    'original': Colorset.original(),
    'peak_load_balance': Colorset.peak_load_balance(),
    'shade_benefit': Colorset.shade_benefit(),
    'shade_benefit_harm': Colorset.shade_benefit_harm(),
    'shade_harm': Colorset.shade_harm(),
    'shadow_study': Colorset.shadow_study(),
    'therm': Colorset.therm(),
    'thermal_comfort': Colorset.thermal_comfort(),
    'view_study': Colorset.view_study()
}


def mesh_to_coordinates(mesh: Mesh2D) -> List[List[List[float], List[float]]]:
    """Convert vertices of Ladybug 2D mesh to coordinates that Plotly can use.

    Args:
        mesh: Ladybug 2D mesh.

    Returns:
        A list of lists of coordinates where each list has two lists. The first has 
        x coordinates and the second has y coordinates.
    """
    cords = []
    for face in mesh.faces:
        x_cords = []
        y_cords = []
        for vert in face:
            x_cords.append(mesh.vertices[vert].x)
            y_cords.append(mesh.vertices[vert].y)
        # Add first cordinate to the end to close the polygon
        x_cords.append(x_cords[0])
        y_cords.append(y_cords[0])
        cords.append([x_cords, y_cords])

    return cords


def verts_to_coordinates(points: List[Point2D], close: bool = True) -> Tuple[List[float], List[float]]:
    """Convert a list of Ladybug Point2Ds to coordinates that Plotly can use.

    Args:
        points: A list of Ladybug Point2Ds.
        close: Boolean to close the polygon. Default: True.

    Returns:
        A tuple of two items

        -   A list of x coordinates.
        -   A list of y coordinates.
    """

    x_cords, y_cords = [], []
    for point in points:
        x_cords.append(point.x)
        y_cords.append(point.y)

    if close:
        x_cords.append(x_cords[0])
        y_cords.append(y_cords[0])

    return x_cords, y_cords


def get_monthly_values(data: MonthlyPerHourCollection) -> List[List[float]]:
    """Get month wise MonthlyPerHour values.

    Args:
        data: A ladybug MonthlyPerHourCollection object.

    Returns:
        A list of lists of monthly values where the length of each list is 24.
    """
    values = []
    for i in range(24, 312, 24):
        values.append([data.values[j] for j in range(i-24, i)])
    return values


def group_monthly(data: List[MonthlyCollection],
                  colors: List[Color]) -> Tuple[List[List[float]], List[List[str]]]:
    """Create monthly lists from values in data and colors.

    Args:
        data: A list of MonthlyCollection objects.
        colors: A list of colors.

    Returns:
        A tuple of two lists:

        -   The is a list of 12 sub-lists. Where each sub-list has the same number of 
            values as the length of data.

        -   The is a list of 12 sub-lists. Where each sub-list has the same number of 
            hex codes as the length of data.
    """

    monthly_values, monthly_colors = [], []
    for i in range(12):
        monthly_values.append([round(item[i], 2) for item in data])
        monthly_colors.append([rgb_to_hex(color) for color in colors])

    return monthly_values, monthly_colors



"""Add capability to generate a Figures from ladybug data objects. """

from typing import List
from plotly.graph_objects import Figure
from ladybug.datacollection import HourlyContinuousCollection, \
    HourlyDiscontinuousCollection, MonthlyCollection, DailyCollection
from ladybug_charts.to_figure import heat_map, monthly_bar_chart, daily_bar_chart, \
    hourly_line_chart, diurnal_average_chart_from_hourly, wind_rose, psych_chart, bar_chart, sunpath,\
    diurnal_average_chart
from ladybug.windrose import WindRose
from ladybug.psychchart import PsychrometricChart
from ladybug.hourlyplot import HourlyPlot
from ladybug.monthlychart import MonthlyChart
from ladybug.sunpath import Sunpath
from ladybug.epw import EPW

HourlyContinuousCollection.heat_map = heat_map
HourlyDiscontinuousCollection.heat_map = heat_map
MonthlyCollection.bar_chart = monthly_bar_chart
DailyCollection.bar_chart = daily_bar_chart
HourlyContinuousCollection.line_chart = hourly_line_chart
HourlyContinuousCollection.diurnal_average_chart = diurnal_average_chart_from_hourly
WindRose.plot = wind_rose
PsychrometricChart.plot = psych_chart
Sunpath.plot = sunpath
EPW.diurnal_average_chart = diurnal_average_chart


def hourly_plot(self, title: str = None, show_title: bool = False,
                num_labels: int = None, labels: List[float] = None) -> Figure:
    hourly_data = self.data_collection
    min_range = self.legend_parameters.min
    max_range = self.legend_parameters.max
    colors = self.legend_parameters.colors
    return heat_map(hourly_data, min_range, max_range, colors, title, show_title,
                    num_labels, labels)


HourlyPlot.plot = hourly_plot


def plot_monthly_chart(self, min_range: float = None, max_range: float = None,
                       title: str = None, show_title: bool = False,
                       stack: bool = False) -> Figure:
    data = self.data_collections
    colors = self.colors
    return bar_chart(data, min_range, max_range, colors, title, show_title, stack)


MonthlyChart.plot = plot_monthly_chart



"""Functions to help charts."""

import plotly.graph_objects as go


def get_dummy_trace(name: str, color: str) -> go:
    """Get a dummy trace to add to a figure."""
    return go.Scatter(
        x=[None],
        y=[None],
        mode='markers',
        marker=dict(size=10),
        showlegend=True,
        line_color=color,
        name=name)



"""Recreating ladybug psychrometric chart using Plotly."""


from ladybug.legend import LegendParameters
import numpy as np
import pandas as pd
import warnings

from math import ceil, floor
from typing import List


import plotly.io as pio
import plotly.graph_objects as go
from plotly.graph_objects import Figure

from ._helper import rgb_to_hex, mesh_to_coordinates, verts_to_coordinates
from .utils import StrategyParameters, Strategy

from ladybug.datacollection import BaseCollection, HourlyContinuousCollection
from ladybug import psychrometrics as psy
from ladybug.psychchart import PsychrometricChart
from ladybug_comfort.chart.polygonpmv import PolygonPMV
from ladybug_geometry.geometry2d.pointvector import Point2D, Vector2D
from ladybug_geometry.geometry2d.line import LineSegment2D
from ladybug_geometry.geometry2d.arc import Arc2D
from ladybug_geometry.geometry2d.polyline import Polyline2D
from ladybug.color import Color

# set white background in all charts
pio.templates.default = 'plotly_white'


def merge_polygon_data(poly_data):
    """Merge an array of polygon comfort conditions into a single data list."""
    val_mtx = [dat.values for dat in poly_data]
    merged_values = []
    for hr_data in zip(*val_mtx):
        hr_val = 1 if 1 in hr_data else 0
        merged_values.append(hr_val)
    return merged_values


def strategy_warning(polygon_name: str) -> str:
    msg = f'Polygon "{polygon_name}" could not fit on the chart given the current'\
        ' location of the comfort polygon(s). Try moving the comfort polygon(s) by'\
        ' changing its criteria to see the missing polygon. \n'
    print(msg)


def _psych_chart(psych: PsychrometricChart, data: BaseCollection = None,
                 title: str = None, show_title: bool = False, polygon_pmv: PolygonPMV = None,
                 strategies: List[Strategy] = [Strategy.comfort],
                 strategy_parameters: StrategyParameters = StrategyParameters(),
                 solar_data: HourlyContinuousCollection = None,
                 colors: List[Color] = None) -> Figure:
    """Create a psychrometric chart.

    Args:
        psych: A ladybug PsychrometricChart object.
        data: A ladybug DataCollection object.
        title: A title for the plot. Defaults to None.
        show_title: A boolean to show or hide the title. Defaults to False.
        polygon_pmv: A ladybug PolygonPMV object. If provided, polygons will be drawn.
            Defaults to None.
        strategies: A list of strategies to be applied to the chart. Accepts a list of
            Stragegy objects. Defaults to out of the box StrategyParameters object.
        strategy_parameters: A StrategyParameters object. Defaults to None.
        solar_data: An annual hourly continuous data collection of irradiance
            (or radiation) in W/m2 (or Wh/m2) that aligns with the data
            points on the psychrometric chart. This is only required when
            plotting a "Passive Solar Heating" strategy polygon on the chart.
            The irradiance values should be incident on the orientation of
            the passive solar heated windows. So using global horizontal
            radiation assumes that all windows are skylights (like a
            greenhouse). Defaults to None.
        colors: A list of colors to be used for the comfort polygons. Defaults to None.

    Returns:
        A plotly figure.
    """

    dbt = psych.temperature
    rh = psych.relative_humidity

    var_range_x = [int(psych.temperature_labels[0]), int(psych.temperature_labels[-1])]
    var_range_y = [0.0, float(psych.hr_labels[-1])]

    # create dummy psych-chart to create mesh
    base_point = Point2D(var_range_x[0], 0)
    psych_dummy = PsychrometricChart(dbt, rh, base_point=base_point, x_dim=1, y_dim=1,
                                     legend_parameters=psych.legend_parameters)

    fig = go.Figure()

    ###########################################################################
    # if no data is provided, plot frequency
    ###########################################################################
    if not data:
        chart_title = 'Psychrometric Chart - Frequency'
        # Plot colored mesh
        cords = mesh_to_coordinates(psych_dummy.colored_mesh)
        for count, cord in enumerate(cords):
            fig.add_trace(
                go.Scatter(
                    x=cord[0],
                    y=cord[1],
                    fill='toself',
                    fillcolor=rgb_to_hex(psych.container.value_colors[count]),
                    line=dict(width=0),
                    showlegend=False,
                    mode='lines',
                )
            )

            # In plotly, it's not possible to have hover text on a filled shape
            # add another trace just to have hover text
            fig.add_trace(
                go.Scatter(
                    x=[psych_dummy.colored_mesh.face_centroids[count].x],
                    y=[psych_dummy.colored_mesh.face_centroids[count].y],
                    showlegend=False,
                    mode='markers',
                    opacity=0,
                    hovertemplate=str(int(psych_dummy.hour_values[count])) + ' hours' +
                    '<extra></extra>',
                )
            )

        # create a dummy trace to make the Legend
        colorbar_trace = go.Scatter(
            x=[None],
            y=[None],
            mode='markers',
            showlegend=False,
            marker=dict(
                colorscale=[rgb_to_hex(color)
                            for color in psych.legend_parameters.colors],
                showscale=True,
                cmin=psych.legend_parameters.min,
                cmax=psych.legend_parameters.max,
                colorbar=dict(thickness=10, title=psych.legend_parameters.title),
            ),
        )
        # add the dummy trace to the figure
        fig.add_trace(colorbar_trace)

    ###########################################################################
    # Load data
    ###########################################################################
    else:
        var = data.header.data_type.name
        chart_title = title if title else f'Psychrometric Chart - {var}'
        lp = LegendParameters(colors=psych_dummy.legend_parameters.colors)
        # add colored data mesh
        mesh, graphic_container = psych_dummy.data_mesh(data, lp)
        cords = mesh_to_coordinates(mesh)
        for count, cord in enumerate(cords):
            fig.add_trace(
                go.Scatter(
                    x=cord[0],
                    y=cord[1],
                    fill='toself',
                    fillcolor=rgb_to_hex(graphic_container.value_colors[count]),
                    line=dict(width=0),
                    showlegend=False,
                    mode='lines'
                ))

            # In plotly, it's not possible to have hover text on a filled shape
            # add another trace just to have hover text
            fig.add_trace(
                go.Scatter(
                    x=[mesh.face_centroids[count].x],
                    y=[mesh.face_centroids[count].y],
                    showlegend=False,
                    mode='markers',
                    opacity=0,
                    hovertemplate=str(
                        int(graphic_container.values[count])) + ' '
                    + graphic_container.legend_parameters.title +
                    '<extra></extra>',
                )
            )

        # create a dummy trace to make the Legend
        colorbar_trace = go.Scatter(
            x=[None],
            y=[None],
            mode='markers',
            showlegend=False,
            marker=dict(
                colorscale=[rgb_to_hex(color)
                            for color in graphic_container.legend_parameters.colors],
                showscale=True,
                cmin=graphic_container.legend_parameters.min,
                cmax=graphic_container.legend_parameters.max,
                colorbar=dict(
                    thickness=10, title=graphic_container.legend_parameters.title),
            ),
        )

        # add the dummy trace to the figure
        fig.add_trace(colorbar_trace)

    ###########################################################################
    # Add lines
    ###########################################################################

    # add relative humidity lines
    for count, polyline in enumerate(psych_dummy.rh_lines):
        # get cordinates from vertices of polygons
        x_cords, y_cords = verts_to_coordinates(polyline.vertices, close=False)
        fig.add_trace(
            go.Scatter(
                x=x_cords,
                y=y_cords,
                showlegend=False,
                mode="lines",
                name="",
                hovertemplate="RH " + psych_dummy.rh_labels[count] + "%",
                line=dict(width=1, color="#85837f"),
            )
        )

    # add enthalpy lines
    for count, line in enumerate(psych_dummy.enthalpy_lines):
        # get cordinates from vertices of polygons
        x_cords, y_cords = verts_to_coordinates(line.vertices, close=False)
        fig.add_trace(
            go.Scatter(
                x=x_cords,
                y=y_cords,
                showlegend=False,
                mode="lines",
                name="",
                hovertemplate="Enthalpy " + psych_dummy.enthalpy_labels[count],
                line=dict(width=1, color="#85837f"),
            )
        )

    # add temperature lines
    for count, line in enumerate(psych_dummy.temperature_lines):
        # get cordinates from vertices of polygons
        x_cords, y_cords = verts_to_coordinates(line.vertices, close=False)
        fig.add_trace(
            go.Scatter(
                x=x_cords,
                y=y_cords,
                showlegend=False,
                mode="lines",
                name="",
                hovertemplate="Temperature " +
                psych_dummy.temperature_labels[count] + ' C',
                line=dict(width=1, color="#85837f"),
            )
        )

    # add humidity ratio lines
    for count, line in enumerate(psych_dummy.hr_lines):
        # get cordinates from vertices of polygons
        x_cords, y_cords = verts_to_coordinates(line.vertices, close=False)
        fig.add_trace(
            go.Scatter(
                x=x_cords,
                y=y_cords,
                showlegend=False,
                mode="lines",
                name="",
                hovertemplate="Humidity Ratio " + psych_dummy.hr_labels[count],
                line=dict(width=1, color="#85837f"),
            )
        )

    ###########################################################################
    # add polygons if requested
    ###########################################################################
    if polygon_pmv:

        # Use colors if provided
        if colors:
            if len(colors) == len(strategies) and\
                    all([isinstance(color, Color) for color in colors]):
                strategy_colors = [rgb_to_hex(color) for color in colors]
            else:
                raise ValueError(
                    'colors must be a list of Color objects and match the'
                    ' length of strategies')
        else:
            # Defult colors for the comfort polygons
            strategy_colors = {
                'Evaporative Cooling': '#008dff',
                'Mass + Night Ventilation': '#333333',
                'Occupant Use of Fans': '#3d17ff',
                'Capture Internal Heat': '#f58700',
                'Passive Solar Heating': '#ff0400',
                'Comfort': '#009402'
            }

        poly_obj = PolygonPMV(psych_dummy)

        # collecting all the polygons
        polygons, polygon_names, polygon_data = [], [], []

        # If other strategies are applied, add their polygons
        if strategies:

            if Strategy.comfort in strategies:
                poly_name = Strategy.comfort.value
                comfort_poly = poly_obj.comfort_polygons[0]
                polygons.append(comfort_poly)
                polygon_names.append(poly_name)
                dat = poly_obj.evaluate_polygon(comfort_poly, tolerance=0.01)
                dat = dat[0] if len(
                    dat) == 1 else poly_obj.create_collection(dat, poly_name)
                polygon_data.append(dat)

            if Strategy.evaporative_cooling in strategies:
                poly_name = Strategy.evaporative_cooling.value
                ec_poly = poly_obj.evaporative_cooling_polygon()
                if ec_poly:
                    polygons.append(ec_poly)
                    polygon_names.append(poly_name)
                    dat = poly_obj.evaluate_polygon(ec_poly, tolerance=0.01)
                    dat = dat[0] if len(
                        dat) == 1 else poly_obj.create_collection(dat, poly_name)
                    polygon_data.append(dat)
                else:
                    strategy_warning(poly_name)

            if Strategy.mas_night_ventilation in strategies:
                poly_name = Strategy.mas_night_ventilation.value
                nf_poly = poly_obj.night_flush_polygon(
                    strategy_parameters.day_above_comfort)
                if nf_poly:
                    polygons.append(nf_poly)
                    polygon_names.append(poly_name)
                    dat = poly_obj.evaluate_night_flush_polygon(
                        nf_poly, psych_dummy.temperature,
                        strategy_parameters.night_below_comfort,
                        strategy_parameters.time_constant, tolerance=0.01)
                    dat = dat[0] if len(
                        dat) == 1 else poly_obj.create_collection(dat, poly_name)
                    polygon_data.append(dat)
                else:
                    strategy_warning(poly_name)

            if Strategy.occupant_use_of_fans in strategies:
                poly_name = Strategy.occupant_use_of_fans.value
                fan_poly = poly_obj.fan_use_polygon(strategy_parameters.fan_air_speed)
                if fan_poly:
                    polygons.append(fan_poly)
                    polygon_names.append(poly_name)
                    dat = poly_obj.evaluate_polygon(fan_poly, tolerance=0.01)
                    dat = dat[0] if len(
                        dat) == 1 else poly_obj.create_collection(dat, poly_name)
                    polygon_data.append(dat)
                else:
                    strategy_warning(poly_name)

            if Strategy.capture_internal_heat in strategies:
                poly_name = Strategy.capture_internal_heat.value
                iht_poly = poly_obj.internal_heat_polygon(
                    strategy_parameters.balance_temperature)
                if iht_poly:
                    polygons.append(iht_poly)
                    polygon_names.append(poly_name)
                    dat = poly_obj.evaluate_polygon(iht_poly, tolerance=0.01)
                    dat = dat[0] if len(
                        dat) == 1 else poly_obj.create_collection(dat, poly_name)
                    polygon_data.append(dat)

            if Strategy.passive_solar_heating in strategies:
                poly_name = Strategy.passive_solar_heating.value
                if not solar_data:
                    warnings.warn('In order to plot a passive solar heating polygon, '
                                  'you need to provide a solar data object.')
                else:
                    bal_t = strategy_parameters.balance_temperature \
                        if Strategy.capture_internal_heat in strategies else None
                    dat, delta = poly_obj.evaluate_passive_solar(
                        solar_data, strategy_parameters.solar_heating_capacity,
                        strategy_parameters.time_constant, bal_t)
                    sol_poly = poly_obj.passive_solar_polygon(delta, bal_t)
                    if sol_poly:
                        polygons.append(sol_poly)
                        polygon_names.append(poly_name)
                        dat = dat[0] if len(
                            dat) == 1 else poly_obj.create_collection(dat, poly_name)
                        polygon_data.append(dat)
                    else:
                        strategy_warning(poly_name)

        else:
            raise ValueError('You need to provide at least one strategy')

        # compute comfrt and total comfort values
        polygon_comfort = [dat.average * 100 for dat in polygon_data] if \
            isinstance(polygon_data[0], BaseCollection) else \
            [dat * 100 for dat in polygon_data]
        if isinstance(polygon_data[0], BaseCollection):
            merged_vals = merge_polygon_data(polygon_data)
            total_comf_data = poly_obj.create_collection(merged_vals, 'Total Comfort')
            total_comfort = total_comf_data.average * 100
        else:
            total_comf_data = 1 if sum(polygon_data) > 0 else 0
            total_comfort = total_comf_data * 100

        # draw each polygon
        for count, polygon in enumerate(polygons):

            # find left , right, top and bottom sides of the polygon
            center_side_dict = {}
            for side in polygon:
                if isinstance(side, LineSegment2D):
                    center_side_dict[side.midpoint] = side
                else:
                    center_side_dict[side.center] = side
            left_key = sorted(list(center_side_dict.keys()), key=lambda x: x.x)[0]
            right_key = sorted(list(center_side_dict.keys()), key=lambda x: x.x)[-1]
            bottom_key = sorted(list(center_side_dict.keys()), key=lambda x: x.y)[0]
            top_key = sorted(list(center_side_dict.keys()), key=lambda x: x.y)[-1]

            # if the top side is a polyline and has 3 vertices
            if isinstance(center_side_dict[top_key], Polyline2D) and \
                    len(center_side_dict[top_key].vertices) == 3:

                # create a center point from the bottom side and a horizontal reference vector
                if isinstance(center_side_dict[bottom_key], LineSegment2D):
                    center_point = center_side_dict[bottom_key].midpoint
                else:
                    center_point = center_side_dict[bottom_key].center
                right_point = Point2D(center_point.x+100, center_point.y)
                ref_vector = LineSegment2D.from_end_points(center_point, right_point).v

                # sort vertices of polyline before creating the arc
                vertices = center_side_dict[top_key].vertices
                vectors = [LineSegment2D.from_end_points(
                    center_point, vert).v for vert in vertices]
                angles = [ref_vector.angle_counterclockwise(vec) for vec in vectors]
                angles_verts_dict = dict(zip(angles, vertices))
                vertices = [angles_verts_dict[angle]
                            for angle in sorted(angles_verts_dict.keys())]
                arc = Arc2D.from_start_mid_end(vertices[2], vertices[1], vertices[0])
                center_side_dict[top_key] = arc.to_polyline(10)

                # sort vertices of polyline created from arc
                top_verts = center_side_dict[top_key].vertices
                vectors = [LineSegment2D.from_end_points(
                    center_point, vert).v for vert in top_verts]
                angles = [ref_vector.angle_counterclockwise(vec) for vec in vectors]
                angles_verts_dict = dict(zip(angles, top_verts))
                anti_clockwise_sorted_top_verts = [angles_verts_dict[angle]
                                                   for angle in sorted(angles_verts_dict.keys())]

                # Collect all the vertices that should be in anti clockwise order
                verts = []
                for vert in center_side_dict[left_key].vertices:
                    verts.append(vert)
                for vert in center_side_dict[bottom_key].vertices:
                    verts.append(vert)
                for vert in center_side_dict[right_key].vertices:
                    verts.append(vert)
                verts += anti_clockwise_sorted_top_verts
            else:
                verts = [point for geo in polygon for point in geo.vertices]

            # get cordinates from vertices of polygons
            x_cords, y_cords = verts_to_coordinates(verts)

            # plot the actual polygons
            fig.add_trace(
                go.Scatter(
                    x=x_cords,
                    y=y_cords,
                    line=dict(
                        width=4,
                        color=strategy_colors[polygon_names[count]] if not colors else strategy_colors[count]),
                    showlegend=True,
                    name=polygon_names[count] + ': ' +
                    str(round(polygon_comfort[count])) + '% of time',
                    mode='lines',
                    hovertemplate='' + '<extra></extra>',
                ))

    # setting the title for the figure
    if show_title:
        fig_title = {
            'text': title if title else chart_title,
            'y': 1,
            'x': 0.5,
            'xanchor': 'center',
            'yanchor': 'top'
        }
    else:
        if title:
            raise ValueError(
                f'Title is set to "{title}" but show_title is set to False.')
        fig_title = None

    fig.update_layout(
        template='plotly_white',
        margin=dict(l=20, r=20, t=33, b=20),
        title=fig_title,
        legend=dict(
            yanchor="top",
            y=0.99,
            xanchor="left",
            x=0.01
        )
    )

    fig.update_xaxes(
        title_text='Temperature (°C)',
        range=var_range_x,
        showline=True,
        linewidth=1,
        linecolor='black',
        mirror=True,
        dtick=5
    )
    fig.update_yaxes(
        title_text='Humidity Ratio (KG water/KG air)',
        range=var_range_y,
        showline=True,
        linewidth=1,
        linecolor='black',
        mirror=True,
    )
    return fig
